<?php
//==>>>==>>>==>>>==>>>==>>>==>>>==>>>==>>>==>>>==>>>==>>>==>>>==>>>
//
// Fade-In Slideshow v2.14 [ FREE TRIAL VERSION ]
// Copyright (c) phpkobo.com ( http://www.phpkobo.com/ )
// Email : admin@phpkobo.com
// ID : FISLS-214
//
//==<<<==<<<==<<<==<<<==<<<==<<<==<<<==<<<==<<<==<<<==<<<==<<<==<<<

class CFieldSet {

	private static $lng_params = array();
	private static $driver_list = array();

	public function __construct( $table ) {
		$this->table = $table;
		$this->fdx = array();
		$this->vals = array();
		$this->attr = array();
		$this->otx = array();
		$this->nsx = array();
		CEnv::loadLang( "fieldset", self::$lng_params );
	}

	public static function pk() {
		return $this->table->getPrimaryKey();
	}

	public static function pkv() {
		return $this->val($this->table->getPrimaryKey());
	}

	public static function isEmptyString( $v ) {
		if ( is_null( $v ) ) {
			return true;
		} else {
			return ( strlen( $v ) == 0 );
		}
	}

	public function isErrorField( $fname ) {
		$name = $this->nsx[ $fname ] . ':' . $fname;
		return CUInfo::exists( "error:{$name}" );
	}

	public function getDname( $fname ) {
		$fd =& $this->table->fdx[ $fname ];
		if ( isset( $fd[ 'dname' ] ) ) {
			return $fd[ 'dname' ];
		} else {
			return $fname;
		}
	}

	public function getRname( $fname ) {
		$fd =& $this->table->fdx[ $fname ];
		if ( isset( $fd[ 'rname' ] ) ) {
			return $fd[ 'rname' ];
		} else {
			return $fname;
		}
	}

	public function getDriver( $name ) {
		if ( !isset( self::$driver_list[ $name ] ) ) {
			$driver = "CFsd_{$name}";
			self::$driver_list[ $name ] = new $driver();
		}
		return self::$driver_list[ $name ];
	}

	public function setRowResult( $result ) {
		$b = false;
		if ( $rs = CSqo::getRowA( $result ) ) {
			foreach( $this->fdx as $fname ) {
				$this->vals[ $fname ] = $rs[ $this->getDname( $fname ) ];
			}
			$b = true;
		}
		return $b;
	}

	public function lng( $key ) {
		return self::$lng_params[ $key ];
	}

	public function clr() {
		$this->vals = array();
		return $this;
	}

	public function val( $key, $val = false ) {
		if ( $val === false ) {
			return isset( $this->vals[$key] ) ?
				$this->vals[$key] : null;
		} else {
			$this->vals[$key] = $val;
		}
	}

	public function att( $key, $val ) {
		$this->attr[$key] = $val;
		return $this;
	}

	public function sel( $selector ) {
		$ch = substr($selector,0,1);
		switch( $ch ) {
		case '+':
			$selector = substr($selector,1);
			$this->fdx = array_merge($this->fdx,$this->table->makeList( $selector ));
			break;
		case '-':
			$selector = substr($selector,1);
			$this->fdx = array_diff($this->fdx,$this->table->makeList( $selector ));
			break;
		default:
			$this->fdx = $this->table->makeList( $selector );
			break;
		}
		return $this;
	}

	public function set( $dx ) {
		foreach( $dx as $key => $val ) {
			$this->vals[ $key ] = $val;
		}
		return $this;
	}

	public function def() {// set default values
		foreach( $this->fdx as $fname ) {
			$fd =& $this->table->fdx[$fname];
			if ( isset( $fd['default'] ) ) {
				$this->val( $fname, $fd['default'] );
			}
		}
		return $this;
	}

	public function sot( $ot, $selector = null ) {// set output type
		if ( is_null( $selector ) ) {
			foreach( $this->fdx as $fname ) {
				$this->otx[ $fname ] = $ot;
			}
		} else {
			foreach( $this->table->makeList( $selector ) as $fname ) {
				$this->otx[ $fname ] = $ot;
			}
		}
		return $this;
	}

	public function sns( $ns, $selector = null ) {// set name space
		if ( is_null( $selector ) ) {
			foreach( $this->fdx as $fname ) {
				$this->nsx[ $fname ] = $ns;
			}
		} else {
			foreach( $this->table->makeList( $selector ) as $fname ) {
				$this->nsx[ $fname ] = $ns;
			}
		}
		return $this;
	}

	public function ruf() {// remove unused fields
		$rx = array();
		foreach( $this->fdx as $fname ) {
			$b = true;
			$fd =& $this->table->fdx[$fname];
			if ( isset($fd['ruffunc']) ) {
				if ( !call_user_func($fd['ruffunc'],$this) ) {
					$b = false;
				}
			}
			if ( $b ) {
				$rx[] = $fname; 
			}
		}
		$this->fdx = $rx;
		return $this;
	}

	public function dump() {
		foreach( $this->vals as $key => $val ) {
			echo "{$key} => {$val}<br/>";
		}
		return $this;
	}

	private static function getModifier( &$s, $sepa, &$modifier ) {
		$pos = strpos( $s, $sepa );
		if ( $pos !== false ) {
			$modifier = substr( $s, 
				strlen( $s ) - ( strlen( $s ) - $pos ) + 1,
				strlen( $s ) );
			$s = substr( $s, 0, $pos );
			return true;
		} else {
			$modifier = null;
			return false;
		}
	}

	private static function mergeFd( &$fd, &$px ) {
		foreach( $px as $key => $val ) {
			if ( is_int( $key ) ) {
				if ( ( $pos = array_search( $val, $fd ) ) !== false ) {
					// do nothing
				} else {
					$fd[] = $val;
				}
			} else if ( $key == 'class' ) {
				if ( isset( $fd[ $key ] ) ) {
					foreach( $val as $k => $v ) {
						if ( is_int( $k ) ) {
							$fd[ $key ][] = $v;
						} else if ( is_null( $v ) ) {
							if ( ( $pos = array_search( $k, $fd[ $key ] ) ) !== false ) {
								unset( $fd[ $key ][ $pos ] );
							}
						}
					}
				} else {
					$fd[ $key ] = $val;
				}
			} else if ( is_array( $val ) ) {
				if ( isset( $fd[ $key ] ) ) {
					self::mergeFd( $fd[ $key ], $val );
				} else {
					$fd[ $key ] = $val;
				}
			} else {
				if ( is_null( $val ) ) {
					if ( ( $pos = array_search( $key, $fd ) ) !== false ) {
						unset( $fd[ $pos ] );
					}
				}
				$fd[ $key ] = $val;
			}
		}
	}

	public function get( $fname, $px = null ) {

		//-- (e.g.) get( "first_name.label!html" );
		self::getModifier( $fname, '!', $output_type );
		self::getModifier( $fname, '.', $data_type );

		//-- does field exist?
		if ( !isset($this->table->fdx[$fname] ) ) {
			trigger_error( "[{$fname}] does not exist in " . 
				"table [{$this->table->name}]", E_USER_ERROR );
		}

		//-- merge px ( onpage configurations )
		if ( is_null( $px ) ) {
			$fd =& $this->table->fdx[$fname];
		} else {
			$fd = $this->table->fdx[$fname];
			self::mergeFd( $fd, $px );
		}

		//-- retrieve output type
		if ( is_null( $output_type ) ) {
			if ( isset( $this->otx[ $fname ] ) ) {
				$output_type = $this->otx[ $fname ];
			} else {
				trigger_error( "[{$fname}] Missing output type", E_USER_ERROR );
			}
		}

		//-- get data
		if ( is_null( $data_type ) ) {

			if ( isset( $fd['input']['type'] ) ) {
				$field_type = $fd['input']['type'];

				//-- [BEGIN] field-type mapping
				switch ( $field_type ) {
				case 'password':
					$field_type = 'text';
					break;
				}
				//-- [END] field-type mapping

			} else {
				$field_type = "text";
			}

			$this->output_type = $output_type;
			$this->field_type = $field_type;

			//-- [BEGIN] mapping for driver
			switch ( $output_type ) {
			case 'table':
				$output_type = 'html';
				break;
			}
			//-- [END] mapping for driver

			return $this->getDriver( "op_{$output_type}_{$field_type}" )
				->exe( $this, $fname, $fd );

		} else {

			if ( isset( $fd[$data_type] ) ) {
				$v = $fd[$data_type];
				switch( $output_type ) {
				case "html": $v = htmlspecialchars($v);
				}
			} else {
				$v = null;
			}

			return $v;
		}
	}

	public function validate( $vtype ) {
		//-- vtype : insert, update, search

		$b_err = false;
		foreach( $this->fdx as $fname ) {
			$fd =& $this->table->fdx[ $fname ];
			if ( !empty( $fd[ 'validator' ] ) ) {
				if ( is_array( $fd[ 'validator' ] ) ) {
					$vx = $fd[ 'validator' ];
				} else {
					$vx = array( $fd[ 'validator' ] );
				}
			} else {
				$vx = array();
			}

			//-- required
			if ( !empty( $fd['required'] ) ) {
				array_unshift( $vx, "required" );
			}

			foreach( $vx as $vd ) {
				$dv = $this->getDriver( "vd_{$vd}" );
				if ( !$dv->exe( $this, $fname, $fd, $vtype ) ) {
					$b_err = true;
					break;
				}
			}
		}

		return !$b_err;
	}

	public function exe( $obj_name, $cmd ) {

		if ( $this->b_report_bool = ( $obj_name[0] == '?' ) ) {
			$obj_name = substr( $obj_name, 1 );
		}

		if ( is_array( $cmd ) ) {

			$this->obj_name = $obj_name;
			$this->cmd =& $cmd;

			$ret = $this->getDriver( "exe_{$obj_name}" )
				->exe( $this, $cmd );

			if ( !$ret && $this->b_report_bool ) {
				return false;
			}

		} else {

			$cx = explode( ";", $cmd );
			foreach( $cx as $c ) {
				$c = trim( $c );

				if ( $c == ">>" ) {
					$c = "get";
				}

				if ( $c == "<<" ) {
					$c = "set";
				}

				$this->obj_name = $obj_name;
				$this->cmd = $c;

				$ret = $this->getDriver( "exe_{$obj_name}_{$c}" )
					->exe( $this );

				if ( !$ret && $this->b_report_bool ) {
					return false;
				}
			}
		}

		if ( $this->b_report_bool ) {
			return true;
		} else {
			return $this;
		}
	}
}

?>