<?php
//==>>>==>>>==>>>==>>>==>>>==>>>==>>>==>>>==>>>==>>>==>>>==>>>==>>>
//
// Fade-In Slideshow v2.14 [ FREE TRIAL VERSION ]
// Copyright (c) phpkobo.com ( http://www.phpkobo.com/ )
// Email : admin@phpkobo.com
// ID : FISLS-214
//
//==<<<==<<<==<<<==<<<==<<<==<<<==<<<==<<<==<<<==<<<==<<<==<<<==<<<

class CSqo {

	public static $conn = null;

	public static function open(
		$DB_HOSTNAME = null,
		$DB_USERNAME = null,
		$DB_PASSWORD = null,
		$DB_DATABASE = null ) {

		if ( !is_null( self::$conn ) ) {
			return self::$conn;
		}

		if ( is_null( $DB_HOSTNAME ) ) {
			$dbs = CDBConf::getDbSettings();
			$DB_HOSTNAME = $dbs[ 'db-hostname' ];
			$DB_USERNAME = $dbs[ 'db-username' ];
			$DB_PASSWORD = $dbs[ 'db-password' ];
			$DB_DATABASE = $dbs[ 'db-database' ];
		}

		self::$conn = @mysqli_connect( $DB_HOSTNAME, $DB_USERNAME, $DB_PASSWORD, $DB_DATABASE );
		if ( $errno = mysqli_connect_errno() ) {
			trigger_error( "Can not connect to database [{$DB_DATABASE}] : " .
				mysqli_connect_error() . " ({$errno})", E_USER_ERROR );
		}

		self::query( "SET NAMES utf8" );

		return self::$conn;
	}

	public static function close() {
		mysqli_close( self::$conn );
		self::$conn = null;
	}

	public static function isEmptyString( $v ) {
		if ( is_null( $v ) ) {
			return true;
		} else {
			return ( strlen( $v ) == 0 );
		}
	}

	public static function sani( $s ) {
		return mysqli_real_escape_string ( self::$conn, $s );
	}

	public static function query( $sql ) {
		if ( is_null( self::$conn ) ) {
			CSqo::open();
		}

		if ( is_array( $sql ) ) {
			$sql = self::renderSql( $sql );
		}

		if ( function_exists( "sys_log" ) ) {
			sys_log( "SQL: " . $sql );
		}

		$result = mysqli_query( self::$conn, $sql );
		if ( !$result ) {
			trigger_error( 'Invalid query: ' .
				mysqli_error( self::$conn ) .
				" (" . mysqli_errno( self::$conn ) . ")",
				E_USER_ERROR );
		}

		return $result;
	}

	public static function getRowCount( $result ) {
		return mysqli_num_rows( $result );
	}

	public static function getRowA( $result ) {
		return mysqli_fetch_array( $result );
	}

	public static function getInsertID() {
		return mysqli_insert_id( self::$conn );
	}

	public static function freeResult( $result ) {
		return mysqli_free_result( $result );
	}

	//----------------------------------------------------------------
	// sql rendering
	//----------------------------------------------------------------
	public static function renderCond( &$wh, $tblas = 'T1' ) {
		if ( empty( $wh['op'] ) ) {

			$qL = '';
			$qR = '';
			switch( $wh[1] ) {

			case "=":
			case "!=":
			case ">=":
			case "<=":
			case ">":
			case "<":
				$sign = $wh[1];
				break;

			case "L%%":
				$sign = 'LIKE';
				$qL = '%';
				$qR = '%';
				break;
			}

			$s = "(";

			$nx = explode(".",$wh[0]);
			if ( count($nx) == 1 ) {
				$name = "`" . $nx[0] . "`";
			} else {
				$name = $nx[0] . ".`" . $nx[1] . "`";
			}
			$s .= $name;

			$s .= " " . $sign . " ";

			if ( $tblas == 'T1' ) {
				if ( is_null( $wh[2] ) ) {
					$s .= 'NULL';
				} else {
					$s .= "'{$qL}" . self::sani( $wh[2] ) . "{$qR}'";
				}
			} else {
				$nx = explode(".",$wh[2]);
				if ( count($nx) == 1 ) {
					$name = "`" . $nx[0] . "`";
				} else {
					$name = $nx[0] . ".`" . $nx[1] . "`";
				}
				$s .= $name;
			}
			$s .= ")";

			return $s;

		} else {
			if ( $wh['op'] == 'NOT' ) {
				$cond = self::renderCond( $wh['cond'] );
				if ( is_null( $cond ) ) {
					return null;
				} else {
					return "(NOT " . self::renderCond( $wh['cond'] ) . ")";
				}
			} else {

				if ( empty( $wh['cond'] ) ) {
					return null;
				} else {
					$cx = array();
					foreach( $wh['cond'] as &$c ) {
						$cond = self::renderCond( $c );
						if ( !is_null( $cond ) ) {
							$cx[] = self::renderCond( $c );
						}
					}
					if ( empty( $cx ) ) {
						return null;
					} else if ( count($cx) == 1 ) {
						return $cx[0];
					} else {
						return "(" . implode( " " . $wh['op'] . " ", $cx ) . ")";
					}
				}
			}
		}
	}

	public static function renderOrderBy( &$ob ) {
		$cx = array();
		foreach( $ob as $key => $val ) {
			$cx[] = "`{$key}` {$val}";
		}
		return implode( ", ", $cx );
	}

	public static function renderSelect( &$ds, $tblas = 'T1' ) {
		$sqlx = array();

		//-- SELECT
		$sqlx[] = "SELECT";

		//-- expr
		if ( !empty( $ds['expr'] ) ) {
			$fx = array();
			foreach( $ds['expr'] as $key => $field ) {
				if ( is_int( $key ) ) {
					$fx[] = "`" . $field . "`";
				} else {
					$fx[] = $key . " AS `" . $field . "`";
				}
			}
			$sqlx[] = implode( ", ", $fx );
		} else {
			$sqlx[] = '*';
		}

		//-- FROM
		$sqlx[] = "FROM";

		//-- table
		$sqlx[] = "`" . $ds['table'] . "` AS {$tblas}" ;

		//-- where
		if ( !empty( $ds['where'] ) ) {
			$where = self::renderCond( $ds['where'], $tblas );
			if ( !is_null( $where ) ) {
				$sqlx[] = "WHERE {$where}";
			}
		}

		//-- order by
		if ( !empty( $ds['orderby'] ) ) {
			$orderby = self::renderOrderBy( $ds['orderby'] );
			if ( !is_null( $orderby ) ) {
				$sqlx[] = "ORDER BY {$orderby}";
			}
		}

		//-- limit
		if ( !empty( $ds['limit'] ) ) {
			$sqlx[] = "LIMIT " . $ds['limit'];
		}

		//-- offset
		if ( !empty( $ds['offset'] ) ) {
			$sqlx[] = "OFFSET " . $ds['offset'];
		}

		return implode( " ",  $sqlx );
	}

	public static function renderInsert( &$ds ) {
		$sqlx = array();

		//-- INSERT
		$sqlx[] = "INSERT";

		//-- INTO
		$sqlx[] = "INTO";

		//-- table
		$sqlx[] = "`" . $ds['table'] . "`" ;

		//-- expr
		$fx = array();
		$dx = array();
		foreach( $ds['data'] as $fname => $val ) {
			$fx[] = "`" . $fname . "`";
			$dx[] = is_null( $val ) ? 'NULL' :
				"'" . self::sani( $val ) . "'";
		}

		$sqlx[] = "(" . implode( ", ", $fx ) . ")";
		$sqlx[] = "VALUES";
		$sqlx[] = "(" . implode( ", ", $dx ) . ")";

		return implode( " ",  $sqlx );
	}

	public static function renderUpdate( &$ds ) {
		$sqlx = array();

		//-- INSERT
		$sqlx[] = "UPDATE";

		//-- table
		$sqlx[] = "`" . $ds['table'] . "`" ;

		//-- SET
		$sqlx[] = "SET";

		//-- expr
		$ax = array();
		foreach( $ds['data'] as $fname => $val ) {
			$ax[] = "`" . $fname . "`" . " = " .
				( is_null( $val ) ? 'NULL' :
				"'" . self::sani( $val ) . "'" );
		}
		$sqlx[] = implode( ", ", $ax );

		//-- where
		if ( !empty( $ds['where'] ) ) {
			$where = self::renderCond( $ds['where'] );
			if ( !is_null( $where ) ) {
				$sqlx[] = "WHERE {$where}";
			}
		}

		return implode( " ",  $sqlx );
	}

	public static function renderDelete( &$ds ) {
		$sqlx = array();

		//-- INSERT
		$sqlx[] = "DELETE FROM";

		//-- table
		$sqlx[] = "`" . $ds['table'] . "`" ;

		//-- where
		$b = false;
		if ( !empty( $ds['where'] ) ) {
			$where = self::renderCond( $ds['where'] );
			if ( !is_null( $where ) ) {
				$sqlx[] = "WHERE {$where}";
				$b = true;
			}
		}

		if ( !$b ) {
			trigger_error( "[CSqo::renderDelete] " . 
				"Missing 'where' clause in DELETE", E_USER_ERROR );
		}

		return implode( " ",  $sqlx );
	}

	public static function renderSql( $ds, $tblas = 'T1' ) {
		switch( $ds['verb'] ) {
		case 'select': return self::renderSelect( $ds, $tblas );
		case 'insert': return self::renderInsert( $ds );
		case 'update': return self::renderUpdate( $ds );
		case 'delete': return self::renderDelete( $ds );
		}
	}
}

?>