<?php
//==>>>==>>>==>>>==>>>==>>>==>>>==>>>==>>>==>>>==>>>==>>>==>>>==>>>
//
// Fade-In Slideshow v2.14 [ FREE TRIAL VERSION ]
// Copyright (c) phpkobo.com ( http://www.phpkobo.com/ )
// Email : admin@phpkobo.com
// ID : FISLS-214
//
//==<<<==<<<==<<<==<<<==<<<==<<<==<<<==<<<==<<<==<<<==<<<==<<<==<<<

class CSys extends CObject {

	const q_prepend = -1;
	const q_call = 0;
	const q_append = 1;
	private $cfg_params = array();

	public function __construct() {
		$this->sys = $this;
		parent::__construct( $this );

		$this->setupCfgParams();

		if ( !isset( $this->cfg_params["max-allowed-events"] ) ) {
			$this->cfg_params["max-allowed-events"] = 100;
		}

		$this->components = array();
		$this->event_queue = array();
		$this->bindings = array();
		$this->event_filters = array();
		$this->log_filters = array();
		$this->b_logging = false;
	}

	private function setupCfgParams() {
		$_cfg =& $this->cfg_params;
		require( $this->getFolderPath() . '/sys/config/config.inc.php' );

		global $sys_params_external;
		if ( isset( $sys_params_external ) ) {
			$_cfg = array_merge( $_cfg, $sys_params_external );
			unset( $sys_params_external );
		}
	}

	public function cfg( $key ) {
		if ( isset( $this->cfg_params[ $key ] ) ) {
			return $this->cfg_params[ $key ];
		} else {
			return '';
		}
	}

	public function getFolderPath() {
		return CEnv::getEntryPath();
	}

	public function getFolderUrl() {
		return CEnv::getEntryUrl();
	}

	public function getAppPath() {
		return dirname(CEnv::getEntryPath()) . '/';
	}

	public function getAppUrl() {
		return dirname(CEnv::getEntryUrl()) . '/';
	}

	public function redirect( $url = null ) {
		if ( is_null($url) ) {
			$url = $this->getFolderUrl();
		}
		header('Location: ' . $url);
		exit;
	}

	private function getComponent( $comp_name ) {
		if ( isset( $this->components[$comp_name] ) ) {
			$comp =& $this->components[$comp_name];
		} else {
			//-- create component
			$path_comp = CEnv::getEntryPath() . "{$comp_name}/";
			$comp = $this->components[$comp_name] = new CComponent( $this, $comp_name, $path_comp );
		}

		return $comp;
	}

	private function processLogFilters( &$entry ) {
		foreach( $this->log_filters as $filter ) {
			if ( method_exists( $filter['object'], $filter['method'] ) ) {
				call_user_func_array(
					array( $filter['object'], $filter['method'] ), array( $entry ) );
			}
		}
	}

	public function isLogging() {
		return $this->b_logging;
	}

	public function log( $entry ) {
		if ( !is_array( $entry ) ) {
			$entry = array(
				'type' => 'console',
				'ln' => $entry,
			);
		}
		$this->processLogFilters( $entry );
	}

	public function registerLogFilter( $obj, $method ) {
		$this->log_filters[] = array(
			'object'=>$obj,
			'method'=>$method,
		);
	}

	public function registerEventFilter( $obj, $method ) {
		$this->event_filters[] = array(
			'object'=>$obj,
			'method'=>$method,
		);
	}

	public function registerBind( $bind, $mode ) {
		$bind->test();

		$cmd_src = "{$bind->comp_name_src}/" . 
			"{$bind->mod_name_src}/" . 
			"{$bind->action_src}";

		if ( !isset( $this->bindings[$cmd_src] ) ) {
			$this->bindings[$cmd_src] = array( null );
		}

		switch ( $mode ) {
		case self::q_prepend:
			array_unshift( $this->bindings[$cmd_src], $bind );
			break;

		case self::q_append:
			$this->bindings[$cmd_src][] = $bind;
			break;
		}
	}

	public function registerEvent( $e, $mode ) {
		$e->test();

		switch ( $mode ) {
		case self::q_prepend:
			array_unshift( $this->event_queue, $e );
			break;

		case self::q_call:
			return $this->processEvent( $e );

		case self::q_append:
			$this->event_queue[] = $e;
			break;
		}
	}

	public function clearEventQueue() {
		$this->event_queue = array();
	}

	private function processEventFilters( $e ) {
		foreach( $this->event_filters as $filter ) {
			if ( method_exists( $filter['object'], $filter['method'] ) ) {
				$e->b_stop = false;
				call_user_func_array(
					array( $filter['object'], $filter['method'] ),
						array( $e ) );
				if ( $e->b_stop ) {
					return false;
				}
			} else {
				trigger_error( "filter method does not exist " . 
					"( method: {$filter['method']} )",
					E_USER_ERROR );
			}
		}
		return true;
	}

	private function setCurrentEvent( $e ) {
		$this->current_event =& $e;
	}

	public function getCurrentEvent() {
		if ( !empty( $this->current_event ) ) {
			return $this->current_event;
		} else {
			return null;
		}
	}

	public function terminate( $exit_code ) {
		//-- 1 : cancel current bindings
		//-- 2 : exit event loop
		//-- 3 : exit event loop and restart
		if ( $exit_code >= 2 ) {
			$this->clearEventQueue();
		}
		$this->exit_code = $exit_code;
	}

	private function processEvent( $e ) {

		if ( $this->event_index > $this->cfg_params["max-allowed-events"] ) {
			trigger_error( 'Too many events', E_USER_ERROR );
		} else {
			$this->event_index++;

			$ret = null;
			$this->exit_code = 0;
			$cmd_src = $e->getCmdPath();
			if ( isset( $this->bindings[$cmd_src] ) ) {
				foreach( $this->bindings[$cmd_src] as $bind ) {
					$b = is_null( $bind );

					if ( $b ) {
						$ee = $e;
					} else {
						$ee = clone $e;
						$ee->cmd_src = $cmd_src;
						$ee->comp_name = $bind->comp_name_des;
						$ee->mod_name = $bind->mod_name_des;
						$ee->action = $bind->action_des;
					}

					if ( $ee->comp_name != '_' ) {
						$this->setCurrentEvent( $ee );
						if ( !$this->processEventFilters( $ee ) ) {
							return $ret;
						}
						$comp = $this->getComponent( $ee->comp_name );
						$ret0 = $comp->processEvent( $ee );

						if ( $b ) {
							$ret = $ret0;
						}

						if ( $this->exit_code >= 1 ) {
							return $ret;
						}
					}
				}
			} else {
				if ( $e->comp_name != '_' ) {
					$this->setCurrentEvent( $e );
					if ( !$this->processEventFilters( $e ) ) {
						return $ret;
					}
					$comp = $this->getComponent( $e->comp_name );
					$ret = $comp->processEvent( $e );
				}
			}

			return $ret;
		}
	}

	public function getPostCommand() {
		return isset( $_REQUEST[ '_cmd' ] ) ?
			$_REQUEST[ '_cmd' ] : null;
	}

	public function clearPostCommand() {
		unset( $_REQUEST[ '_cmd' ] );
	}

	private function usePostCommand() {
		$cmdx = explode( ";", $_REQUEST['_cmd' ] );
		foreach( $cmdx as $cmd ) {
			$cmd = trim( $cmd );
			$cx = explode( "/", $cmd );
			if ( count( $cx ) < 3 ) {
				if ( !isset( $_REQUEST['_cmd_path' ] ) ) {
					trigger_error( 'missing _cmd_path parameter', E_USER_ERROR );
				}
				$cpx = explode( "/", $_REQUEST['_cmd_path' ] );
				if ( count( $cpx ) != 3 ) {
					trigger_error( 'invalid _cmd_path parameter', E_USER_ERROR );
				}
				switch ( count( $cx ) ) {
				case 1:
					$cpx[2] = $cx[0];
					break;
				case 2:
					$cpx[2] = $cx[1];
					$cpx[1] = $cx[0];
					break;
				}
				$cmd = implode( "/", $cpx );
			}
			$e = new CEvent( $this );
			$e->add( $cmd );
		}
	}

	public function setupCommands( $cmd ) {
		if ( !is_array( $cmd ) ) {
			$cmd = array( $cmd );
		}

		foreach( $cmd as $c ) {
			if ( $c[0] == '!' ) { 
				if ( isset( $_REQUEST[ '_cmd' ] ) ) {
					$this->usePostCommand();
					continue;
				} else {
					$c = substr( $c, 1 );
				}
			}

			$e = new CEvent( $this );
			$e->add( $c );
		}
	}

	public function runEventLoop() {
		while( $e = array_shift( $this->event_queue ) ) {
			$this->processEvent( $e );
			if ( $this->exit_code >= 2 ) {
				return;
			}
		}
	}

	public function run() {
		$this->event_index = 0;

		//-- dev functions
		$path = CEnv::cfg("dev");
		if ( !empty( $path ) && file_exists( $path ) ) {
			include_once( $path );
		}

		//-- event loop
		do {
			$this->exit_code = 0;
			$this->setupCommands( $this->cfg_params["startup"] );
			$this->runEventLoop();
		} while( $this->exit_code == 3 );

		//-- exit
		if ( $this->isLogging() ) {
			$this->log( "::exit" );
		}
	}
}

?>